#---------------------------------------------------------------------------
#
#   MWG - Level View
#
#---------------------------------------------------------------------------

from pygame import Rect, draw
from albow.widget import Widget
from albow.image_array import get_image_array
from tiles import tile_images
from constants import *
from moving_objects import YarnBall

yarn_map = {
	YARN_EW: 0,
	YARN_NS: 1,
	YARN_SE: 2,
	YARN_SW: 3,
	YARN_NW: 4,
	YARN_NE: 5,
}

yarn_tiles = get_image_array("yarn_segments.png", len(yarn_map), border = 2)
yarn_end_images = get_image_array("yarn_ends.png", 4, border = 2)

class LevelView(Widget):

	window_size = (10, 9)  # in cells
	cell_size = (80, 60)    # in pixels
	
	show_cell_grid = False

	def __init__(self, game):
		ww, wh = self.window_size
		cw, ch = self.cell_size
		rect = Rect(0, 0, ww * cw, wh * ch)
		Widget.__init__(self, rect)
		self.game = game
	
	def level(self):
		return self.game.playing_level

	def get_window_pos(self):
		return self.level().window_pos
	
	def set_window_pos(self, p):
		self.level().window_pos = p
	
	def draw(self, surf):
		self.adjust_window()
		level = self.level()
		lw, lh = level.level_size
		ww, wh = self.window_size
		cw, ch = self.cell_size
		i0, j0 = level.window_pos
		i1, j1 = min(i0 + ww, lw), min(j0 + wh, lh)
		cell_objects = [[[] for j in xrange(j0, j1)]
			for i in xrange(i0, i1)]
		for obj in level.moving_objects:
			i, j = obj.cell_coords()
			if i0 <= i < i1 and j0 <= j < j1:
				cell_objects[i - i0][j - j0].append(obj)
		def itercells():
			for i in xrange(i0, i1):
				r = Rect((i - i0) * cw, (j - j0) * ch, cw, ch)
				cell = level.cells[i][j]
				yield i, r, cell
		for j in xrange(j0, j1):
			for i, r, cell in itercells():
				self.draw_ground(surf, r, cell)
				self.draw_pavement(surf, r, cell)
				for obj in cell_objects[i - i0][j - j0]:
					if isinstance(obj, YarnBall):
						centre = self.object_coords(obj, i0, j0)
						self.draw_yarn_end(surf, r, obj, centre)
						if obj is cell.object_falling_in:
							obj.draw(surf, centre)
				if cell.object_falling_in:
					self.draw_ground_front(surf, r, cell)
				self.draw_yarn(surf, r, cell)
		for j in xrange(j0, j1):
			for i, r, cell in itercells():
				self.draw_obstacle(surf, r, cell)
				self.post_draw_cell(surf, cell, r)
			objects = level.moving_objects_in_row(j)
			objects.sort(key = lambda obj: obj.position[1])
			for obj in objects:
				if not obj.falling_into_cell:
					obj.draw(surf, self.object_coords(obj, i0, j0))
					self.post_draw_object(surf, obj)
	
	def draw_ground(self, surf, r, cell):
		self.draw_tile(surf, r, cell.ground)
		if self.show_cell_grid:
			draw.rect(surf, self.grid_color, r, 1)
	
	def draw_pavement(self, surf, r, cell):
		self.draw_tile(surf, r, cell.pavement)
	
	def draw_yarn(self, surf, r, cell):
		for yarn in cell.yarn:
			index = yarn_map[yarn]
			image = yarn_tiles[index]
			surf.blit(image, r)
	
	def draw_obstacle(self, surf, r, cell):
		self.draw_tile(surf, r, cell.obstacle)
	
	def draw_ground_front(self, surf, r, cell):
		ground = cell.ground
		if ground:
			self.draw_tile(surf, r, ground + "-front")
	
#	def draw_moving_objects(self, surf, cell, objects, i0, j0):
#		for obj in objects:
#			if not obj.falling_into_cell:
#				obj.draw(surf, self.object_coords(obj, i0, j0))
#				self.post_draw_object(surf, obj)
	
	def post_draw_object(self, surf, obj):
		pass
	
	def post_draw_cell(self, surf, cell, r):
		pass
	
	def object_coords(self, obj, i0, j0):
		cw, ch = self.cell_size
		px, py = obj.position
		x = int((px - i0) * cw)
		y = int((py - obj.z - j0) * ch)
		return (x, y)

	def draw_tile(self, surf, frame, name):
		if name:
			image = tile_images.get(name)
			if image:
				r = image.get_rect()
				r.midbottom = frame.midbottom
				surf.blit(image, r)

	def draw_yarn_end(self, surf, cell_rect, obj, center):
		dir = obj.yarn_dir
		if dir is not None:
			cx, cy = cell_rect.center
			image = obj.get_frame()
			obj_rect = obj.get_frame_rect(image, center)
			ox, oy = obj_rect.center
			dest_rect = Rect(cell_rect)
			#print "LevelView.draw_yarn_end:" ###
			#print "...cell rect =", cell_rect ###
			#print "...obj rect  = ", obj_rect ###
			link_dest = yarn_end_images.get_rect()
			dest_rect = None
			if dir == DIR_EAST or dir == DIR_WEST:
				image = yarn_tiles[yarn_map[YARN_EW]]
				if dir == DIR_WEST:
					if obj_rect.left > cell_rect.left:
						dest_rect = Rect(cell_rect.left, cell_rect.top,
							obj_rect.left - cell_rect.left, cell_rect.height)
					link_dest.midleft = (max(obj_rect.left, cell_rect.left), cy)
					if oy < cy:
						link = 0
					else:
						link = 1
				else:
					if obj_rect.right < cell_rect.right:
						dest_rect = Rect(obj_rect.right, cell_rect.top,
							cell_rect.right - obj_rect.right, cell_rect.height)
					link_dest.midright = (min(obj_rect.right, cell_rect.right), cy)
					if oy < cy:
						link = 2
					else:
						link = 3
			else:
				image = yarn_tiles[yarn_map[YARN_NS]]
				if dir == DIR_NORTH:
					if obj_rect.top > cell_rect.top:
						dest_rect = Rect(cell_rect.left, cell_rect.top,
							cell_rect.width, obj_rect.top - cell_rect.top)
					#print "...doing link north" ###
					link_dest.midtop = (cx, max(obj_rect.top, cell_rect.top))
					if ox < cx:
						link = 0
					else:
						link = 2
				else:
					if obj_rect.bottom < cell_rect.bottom:
						dest_rect = Rect(cell_rect.left, obj_rect.bottom,
							cell_rect.width, cell_rect.bottom - obj_rect.bottom)
					link_dest.midbottom = (cx, min(obj_rect.bottom, cell_rect.bottom))
					if ox < cx:
						link = 1
					else:
						link = 3
			if dest_rect:
				src_rect = Rect((0, 0), dest_rect.size)
				#print "...dest rect =", dest_rect ###
				#print "...src rect  =", src_rect ###
				surf.blit(image, dest_rect, src_rect)
			#print "...link dest =", link_dest ###
			#draw.rect(surf, (0, 0, 255), link_dest) ###
			#print "link no =", link ###
			link_image = yarn_end_images[link]
			src_rect = Rect((0, 0), link_dest.size)
			surf.blit(link_image, link_dest, src_rect)

	def adjust_window(self):
		wx, wy = self.get_window_pos()
		ww, wh = self.window_size
		x, y = self.level().player.position
		x0 = int(x - 0.5)
		y0 = int(y - 0.5)
		x1 = int(x + 0.5)
		y1 = int(y + 0.5)
		if x0 < wx:
			wx = x - ww + 1
		elif x1 >= wx + ww:
			wx = x
		if y0 < wy:
			wy = y - wh - 1
		elif y1 >= wy + wh:
			wy = y
		self.move_window_to((int(wx), int(wy)))
	
	def move_window_to(self, (wx, wy)):
		lw, lh = self.level().level_size
		ww, wh = self.window_size
		wx = max(0, min(wx, lw - ww))
		wy = max(0, min(wy, lh - wh))
		#print "Level: moving window to", (wx, wy) ###
		self.set_window_pos((wx, wy))
