#---------------------------------------------------------------------------
#
#   MWG - Moving objects
#
#---------------------------------------------------------------------------

from __future__ import division
import pygame.locals as pk
from albow.image_array import get_image_array
from constants import *
from utils import opposite_dir

key_to_direction = {
	pk.K_RIGHT: DIR_EAST,
	pk.K_DOWN: DIR_SOUTH,
	pk.K_LEFT: DIR_WEST,
	pk.K_UP: DIR_NORTH,
}

#---------------------------------------------------------------------------

class MovingObject(object):

	pushable = False
	fall_rate = 1/8
	max_fall_distance = 2
	
	falling_into_cell = None
	z = 0
	
	def get_frame_size(self):
		frame = self.get_frame()
		return frame.get_size()

	def cell_coords(self):
		p = self.position
		return int(p[0]), int(p[1])

	def is_in_cell(self, coords):
		cx, cy = coords
		x, y = self.position
		return cx <= x < cx+1 and cy <= y < cy+1

	def draw(self, surf, coords):
		image = self.get_frame()
		if image:
			r = self.get_frame_rect(image, coords)
			surf.blit(image, r)
	
	def get_frame_rect(self, image, coords):
		x, y = coords
		r = image.get_rect()
		r.midbottom = x, y + r.height // 3
		return r

	def begin_frame(self):
		if self.falling_into_cell:
			self.z -= self.fall_rate
			if self.z < -self.max_fall_distance:
				self.level.remove_moving_object(self)
				self.fallen_down_hole()

	def hit_by_movement(self, other, dir, dist):
		if self.falling_into_cell:
			return False
		x0, y0 = self.position
		x1, y1 = other.position
		r = self.radius + other.radius
		#print "MovingObject.hit_by_movement:", self, other, dir, dist ###
		#print "...self pos =", (x0, y0) ###
		#print "...other pos =", (x1, y1) ###
		#print "...radius =", r ###
		if dir == DIR_EAST:
			#print "...checking east" ###
			h = y1 - r < y0 < y1 + r and x1 < x0 and x1 + r + dist > x0
		elif dir == DIR_SOUTH:
			h = x1 - r < x0 < x1 + r and y1 < y0 and y1 + r + dist > y0
		elif dir == DIR_WEST:
			h = y1 - r < y0 < y1 + r and x1 > x0 and x1 - r - dist < x0
		else: # DIR_NORTH
			h = x1 - r < x0 < x1 + r and y1 > y0 and y1 - r - dist < y0
		return h
	
	def can_move_into_cell(self, dir, cell):
		return True

	def change_position(self, dir, p):
		old_cell_coords = self.cell_coords()
		self.position = p
		new_cell_coords = self.cell_coords()
		self.moved(dir)
		if old_cell_coords != new_cell_coords:
			level = self.level
			old_cell = level.cell_at(old_cell_coords)
			new_cell = level.cell_at(new_cell_coords)
			self.changed_cell(dir, old_cell, new_cell)
	
	def moved(self, dir):
		pass
	
	def changed_cell(self, dir, old_cell, new_cell):
		pass
	
	def fallen_down_hole(self):
		pass

#---------------------------------------------------------------------------

class YarnBall(MovingObject):

	num_frames = 12
	frames = get_image_array("yarnball.png", (2, num_frames))
	radius = 1/4
	pushable = True
	
	#  direction --> (animation, delta_phase)
	roll_map = [(0, 1), (1, 1), (0, -1), (1, -1)]
	
	position = (2.5, 4.5)
	animation = 0
	phase = 0
	yarn_dir = None  # direction of unrolled yarn
	
	def get_frame(self):
		return self.frames[self.animation, self.phase]

	def can_move_into_cell(self, dir, cell):
		if dir == self.yarn_dir:
			if cell.yarn_submerged(opposite_dir(dir)):
				return False
		return True

	def moved(self, dir):
		a, dp = self.roll_map[dir]
		self.animation = a
		self.phase = (self.phase + dp) % self.num_frames 

	def changed_cell(self, dir, old_cell, new_cell):
		yarn_dir = self.yarn_dir
		if yarn_dir is not None:
			if dir == yarn_dir:
				self.yarn_dir = new_cell.remove_yarn(opposite_dir(dir))
			else:
				old_cell.add_yarn(yarn_dir, dir)
				self.yarn_dir = opposite_dir(dir)
		if new_cell.is_hole and not new_cell.yarn and self.yarn_dir is None:
			self.falling_into_cell = new_cell
			new_cell.object_falling_in = self
			x, y = self.position
			self.position = int(x) + 0.5, int(y) + 0.5

	def fallen_down_hole(self):
		self.level.finish()

#---------------------------------------------------------------------------

ACT_STAND = 0
ACT_WALK = 1

class Player(MovingObject):

	first_walk_frame = 1
	num_walk_frames = 8
	num_frames = 9
	step_size = 1/8
	radius = 1/4
	
	frames = [
		get_image_array("myna_%s.png" % dir, num_frames, border=2)
			for dir in ("east", "south", "west", "north")]
	
	position = (4.5, 4.5)
	facing = DIR_EAST
	action = ACT_STAND
	phase = 0
	stop_movement_pending = None
	
	def get_frame(self):
		frames = self.frames[self.facing]
		act = self.action
		if act == ACT_STAND:
			return frames[0]
		elif act == ACT_WALK:
			return frames[self.first_walk_frame + self.phase]

	def begin_frame(self):
		act = self.action
		if act == ACT_WALK:
			if self.move_in_direction(self.facing):
				self.phase = (self.phase + 1) % self.num_walk_frames
		stop_dir = self.stop_movement_pending
		if stop_dir is not None:
			if stop_dir == self.facing:
				self.action = ACT_STAND
			self.stop_movement_pending = None
	
	def move_in_direction(self, dir):
		#print "Player.move_in_direction:", dir ###
		return self.level.try_to_move_object(self, dir, self.step_size)

	def key_down(self, e):
		k = e.key
		dir = key_to_direction.get(k) 
		if dir is not None:
			self.start_moving_cmd(dir)

	def key_up(self, e):
		k = e.key
		dir = key_to_direction.get(k) 
		if dir is not None:
			self.stop_moving_cmd(dir)
	
	def start_moving_cmd(self, dir):
		#print "Player.start_moving_cmd:", dir ###
		self.facing = dir
		act = self.action
		if act == ACT_STAND:
			self.action = ACT_WALK
			self.phase = 0
	
	def stop_moving_cmd(self, dir):
		self.stop_movement_pending = dir
